/*
 * JBoss, Home of Professional Open Source
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags.
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * (C) 2009
 * @author JBoss Inc.
 */
package org.jboss.jbossts.rts.demo.client.service;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.ClickListener;

// making client side HTTP requests
import com.google.gwt.http.client.URL;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;

//JSNI
import com.google.gwt.core.client.JavaScriptObject;

import com.google.gwt.core.client.JsArray;

import java.util.Map;
import java.util.HashMap;

public class ServicePanel extends VerticalPanel
{
	private String svcUrl;
	private String name;
	private Label siteUrl = new Label();
	private Label status = new Label();
	private Panel panel = new HorizontalPanel();
	private Map<String, WidgetBinding> widgets = new HashMap<String, WidgetBinding> ();

	protected StatusPanel summary;

	private static int jsonRequestId = 0;

	public ServicePanel(String name, String title, String svcUrl, StatusPanel summary)
	{
		this.name = name;
		this.svcUrl = svcUrl;
		this.summary = summary;

		Label titleLabel = new Label(title);
		titleLabel.addStyleName("h2");
		super.add(titleLabel);
		super.add(panel);
	}

	public String getName()
	{
		return name;
	}

	public void add(Widget child)
	{
		panel.add(child);
	}

	protected void addWidgetBinding(String property, WidgetBinding widget)
	{
		widgets.put(property, widget);
	}

	protected void fini()
	{
		addButtons(panel);

		siteUrl.setText("External URL: " + svcUrl);
		siteUrl.addStyleName("message");
		status.addStyleName("message");
		super.add(siteUrl);
		super.add(status);
	}

	protected void addButtons(Panel panel)
	{
		Button confirm = new Button("Modify Booking");
		Button refresh = new Button("Refresh");

		panel.add(confirm);
//		panel.add(refresh);

		confirm.addStyleDependentName("update");
		confirm.addClickListener(new ClickListener() {
			public void onClick(Widget event) {
					update();
			}
		});
		refresh.addStyleDependentName("update");
		refresh.addClickListener(new ClickListener() {
				public void onClick(Widget event) {
						refresh();
				}
		});
	}

	protected void update()
	{
		StringBuilder q = new StringBuilder("?txId=");

		q.append(summary.getTxId());
		q.append("&service=");
		q.append(name);

		for (Map.Entry<String, WidgetBinding> binding : widgets.entrySet())
		{
			String name = binding.getKey();
			String value = binding.getValue().getValue();

			if (value != null && !name.equals("status")) {
				q.append("&").append(binding.getKey()).append("=").append(value);
			}
		}

		//com.google.gwt.user.client.Window.alert(q.toString());
		sendRequest(q.toString());
	}

	public void refresh()
	{
		reset();

		StringBuilder q = new StringBuilder("?txId=");

		q.append(summary.getTxId());
		q.append("&service=");
		q.append(name);

		sendRequest(q.toString());
	}

	public void reset()
	{
		for (WidgetBinding binding : widgets.values()) {
			binding.reset();
			binding.onSuccess(binding.getValue());
		}
	}

	private void displayStatus(String msg)
	{
		status.setText(msg);
	}

	private void displayError(String msg)
	{
		status.setText(msg);
	}

	private static synchronized int nextId() { return jsonRequestId++; }

	protected void sendRequest(final String url)
	{
		String u = URL.encode(svcUrl + url) + "&callback=";
		getJson(nextId(), u, this);
	}

	public native static void getJson(int requestId, String url, ServicePanel handler) /*-{
		var callback = "callback" + requestId;

		// [1] Create a script element.
		var script = document.createElement("script");
		script.setAttribute("src", url+callback);
		script.setAttribute("type", "text/javascript");

		// [2] Define the callback function on the window object.
		window[callback] = function(jsonObj) {
		// [3]
			handler.@org.jboss.jbossts.rts.demo.client.service.ServicePanel::handleJsonResponse(Lcom/google/gwt/core/client/JavaScriptObject;)(jsonObj);
			window[callback + "done"] = true;
		}
  
		// [4] JSON download has 1-second timeout.
		setTimeout(function() {
			if (!window[callback + "done"]) {
					handler.@org.jboss.jbossts.rts.demo.client.service.ServicePanel::handleJsonResponse(Lcom/google/gwt/core/client/JavaScriptObject;)(null);
			}
  
				// [5] Cleanup. Remove script and callback elements.
				document.body.removeChild(script);
				delete window[callback];
				delete window[callback + "done"];
		}, 1000);

		// [6] Attach the script element to the document body.
		document.body.appendChild(script);
}-*/;

	public void handleJsonResponse(JavaScriptObject jso) {
		if (jso == null) {
			displayError("Error retrieving JSON");
			//com.google.gwt.user.client.Window.alert("no JSON response");
		} else {
			JsArray<BookingData> data = asArrayOfBookingData(jso);
			StringBuilder s = new StringBuilder();
			try {
				for (int i = 0; i < data.length(); i++) {
					BookingData bd = data.get(i);
					bd.toString(s);
					bd.updateWidgets(widgets);
				}
			} catch (Throwable t) {
				//com.google.gwt.user.client.Window.alert("overlay error: " + t.getMessage());
				displayError("JSON overylay error: " + t.getMessage());
			}

			displayStatus(s.toString());
			summary.update();
		}
	}

	// JSON string to JavaScript
	private final native JsArray<BookingData> asArrayOfBookingData(JavaScriptObject jso) /*-{
		return jso;
	}-*/;

	public static ListBox createListBox(boolean dropdown, String style, int selectedIndex, String[] items)
	{
		ListBox widget = new ListBox();

		if (style != null)
			widget.addStyleName(style);
		for (int i = 0; i < items.length; i++)
			widget.addItem(items[i]);

		if(!dropdown)
			widget.setVisibleItemCount(3);

		if (selectedIndex >= 0)
			widget.setSelectedIndex(selectedIndex);

		return widget;
	}

}
