/*
 * JBoss, Home of Professional Open Source
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags.
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * (C) 2009
 * @author JBoss Inc.
 */
package org.jboss.jbossts.rts.demo.client.service;

import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.HTML;

public class StatusPanel extends VerticalPanel
{
	private String STATUS_PATERN = ""
+ "<h2>Booking Status: <b style=\"color:#fff\">#STATUS</b></h2>\n"
+ "<b>Booking Id</b>: #ID<br>\n"
+ "<b>Theatre</b>: #SEATS seats in the #TYPE (Status: <b style=\"color:#fff\">#THEATRE_STATUS</b>)<br>\n"
+ "<b>Restaurant</b>: table for #DINERS people (Status: <b style=\"color:fff\">#RESTAURANT_STATUS</b>)<br>\n"
+ "<b>Taxi</b>: #TAXI (Status: <b style=\"color:fff\">#TAXI_STATUS</b>)\n";
	private HTML STATUS_HTML = new HTML();

	private int seats = 0;
	private String seatType = "Stalls";
	private int diners = 0;
	private boolean taxi = false;
	private String txId;
	private String status = "";
	private String id = "";
	private String theatreStatus = "Unknown";
	private String restaurantStatus = "Unknown";
	private String taxiStatus = "Unknown";

	public StatusPanel(int seats, String seatType, int diners, boolean taxi)
	{
				add(STATUS_HTML);
				addStyleName("addPanel");
				setVisible(true);
		}

		private void updateStatus()
		{
				STATUS_HTML.setHTML(STATUS_PATERN
						.replace("#THEATRE_STATUS", theatreStatus)
						.replace("#RESTAURANT_STATUS", restaurantStatus)
						.replace("#TAXI_STATUS", taxiStatus)
						.replace("#ID", id)
						.replace("#SEATS", String.valueOf(seats))
						.replace("#TYPE", seatType)
						.replace("#DINERS", String.valueOf(diners))
						.replace("#TAXI", taxi ? "required" : " not required")
						.replace("#STATUS", status));
		}

	private void reset()
	{
		seats = 0;
		seatType = "";
		diners = 0;
		taxi = false;
		status = "";
		theatreStatus = "Unknown";
		restaurantStatus = "Unknown";
		taxiStatus = "Unknown";
	}

	public void setStatus(String status)
	{
		if (status != null)
			this.status = status;

		updateStatus();
	}

	public void clearStatus(boolean isComplete)
	{
		reset();
		setStatus(isComplete ? "Confirmed" : "Cancelled");
	}

	public void clearStatus(String status)
	{
		reset();
		setStatus(status);
	}

	private boolean in_same_state(String state)
	{
		return (theatreStatus.equals(state) &&
			restaurantStatus.equals(state) &&
			taxiStatus.equals(state));
	}

	private boolean any_in_state(String state)
	{
		return (theatreStatus.equals(state) ||
			restaurantStatus.equals(state) ||
			taxiStatus.equals(state));
	}

	public boolean is_complete()
	{
		return "Booked".equals(status) || "Cancelled".equals(status);
	}

	public String getStatus(String service)
	{
		if (service.equals(theatreStatus))
			return theatreStatus;
		else if (service.equals(restaurantStatus))
			return restaurantStatus;
		else if (service.equals(taxiStatus))
			return taxiStatus;

		return "Unknown";
	}
	public boolean is_complete(String service)
	{
		String stat = getStatus(service);;

		return "commit_ok".equals(stat) || "rollback_ok".equals(stat);
	}

	public boolean is_in_progress(String service)
	{
		String stat = getStatus(service);;

		return "prepare_ok".equals(status) || "Cancelling".equals(status);
	}

	public void update()
	{
		if (in_same_state("commit_ok"))
			setStatus("Booked");
		else if (in_same_state("rollback_ok"))
			setStatus("Cancelled");
		else if (any_in_state("rollback_ok"))
			setStatus("Partially Cancelled");
		else if (any_in_state("commit_ok"))
			setStatus("Partially Booked");
		else if (any_in_state("prepare_ok"))
			setStatus("Confirming");
		else if (any_in_state("New"))
			setStatus("Unconfirmed");
		else
			setStatus("Taxi: " + taxiStatus + " Theatre: " + theatreStatus + " Restaurant: " + restaurantStatus);
	}

	public void updateTheatre(int seats, String seatType)
	{
		this.seats = seats;
		this.seatType = seatType;
		setStatus("Unconfirmed");
	}

	public void updateTheatre(int seats)
	{
		this.seats = seats;
		setStatus("Unconfirmed");
	}

	public void updateTheatre(String seatType)
	{
		this.seatType = seatType;
		setStatus("Unconfirmed");
	}

	public void updateTheatreStatus(String status)
	{
		this.theatreStatus = status;
		setStatus("Unconfirmed");
	}

	public void updateRestaurant(int diners)
	{
		this.diners = diners;
		setStatus("Unconfirmed");
	}

	public void updateRestaurantStatus(String status)
	{
		this.restaurantStatus = status;
		setStatus("Unconfirmed");
	}

	public void updateTaxi(boolean required)
	{
		this.taxi = required;
		setStatus("Unconfirmed");
	}

	public void updateTaxiStatus(String status)
	{
		this.taxiStatus = status;
		setStatus("Unconfirmed");
	}

	public String getTxId()
	{
		return txId;
	}

	public String getId()
	{
		id = txId.substring(txId.lastIndexOf('/') + 1);
		return id;
	}

	public void setTxId(String txId)
	{
		this.txId = txId;
		this.id = txId.substring(txId.lastIndexOf('/') + 1);

		updateStatus();
	}
}
